<?php

/* 
 * dbquery.inc - the script performing the database query and creating the HTML output for it
 *
 * Copyright (c) by Andreas S. Krebs (andreas.krebs@web.de)
 *
 * this script expects in the variable $query the WHERE clause of the SQL query to perform
 * and returns in $entries_found the number of entries found in the database.
 *
 */

require_once("config.inc");
require_once("utils.inc");
require_once("dbutils.inc");

function createSearch_TABLE($query, $notfound_msg, $isSearch)
{
  global $DBCONNECT, $TABLE_BORDER, $MAX_ENTRIES, $SHOW_COLUMN, $QUERY_STRING, $HAS_JS, $SCRIPT_NAME, $LINKROOT, $MP3ROOT, $ENTRY_ATTR;
  global $conn, $sortby, $selEntries, $showsel, $addentries, $start;

  $table1=new TABLEtag(array("CELLSPACING"=>"4", 
			     "CELLPADDING"=>"0",
			     "BORDER"=>"$TABLE_BORDER",
			     "WIDTH"=>"100%"));

  if(!$conn)	// recycle connection if already existing
    $conn=pg_pconnect($DBCONNECT);
  if(!$conn)
  {
    $table1->push_row(create_TD(array("CLASS"=>"Navigation", "WIDTH"=>"100%", "ALIGN"=>"CENTER"),
				html_b("Failed to open the database \"$DBCONNECT\"")));
  }
  else
  {
    // Execute query (we also get the oid, also we don't need it yet)
    if($sortby == "")
      $sortby="artist ASC,title ASC,album ASC";

    $sortdir="ASC";
    $isortdir="DESC";
    if(stristr($sortby, "ASC") === false)
    {
      $sortdir="DESC";
      $isortdir="ASC";
    }

    if(is_array($selEntries)==false)
      $selEntries=array();

    if($showsel==1)
      $query=addSelEntriesToQuery($query, $selEntries);

   if($query!="")
      $result=pg_exec($conn, createSongQuery("*", $query, $sortby));

    $entries_found=pg_numrows($result);

    if($addentries==1)
    {
        for($i=0; $i<$entries_found; $i++)
        {
          $sid=pg_fetch_result($result, $i, "id");
          if(!in_array($sid, $selEntries))
            $selEntries[]=$sid;
        }
    }
 
    if($start >= $entries_found)		// make sure the start range is valid
      $start=0;

    if($entries_found <= 0)
    {
      $table1->push_row(create_TD(array("CLASS"=>"Navigation", "WIDTH"=>"100%", "ALIGN"=>"CENTER"),
			html_b("$notfound_msg")));

      if(count($selEntries)>0)
        $table1->push_row(create_TD(array("CLASS"=>"Navigation", "WIDTH"=>"100%", "ALIGN"=>"CENTER"),
			  html_b(count($selEntries)." entries still selected")));
    }
    else
    {
      $end=$start + $MAX_ENTRIES;
      if($end > $entries_found)
        $end=$entries_found;

      $colspan=0;
      while(list($col, $show)=each($SHOW_COLUMN))
      {
        if($show)
          $colspan++;
      }

      $playpre=$prefix=$postfix=$pleditpre="";
      $playTarget="TARGET=\"player\"";
      if($isSearch || $entries_found>1)
      {
        $colspan++;

        $newquery=$QUERY_STRING;
        if($HAS_JS)
        {
          $playTarget=NULL;
          $prefix="javascript:showMoreEntries('";
          $postfix="')";
          $playpre="javascript:playerComand('";
          $pleditpre="javascript:blankComand('";

          $newquery=preg_replace("/&i_\d*=1/" , "", $newquery);		// remove all entries
          $newquery=ereg_replace("&addentries=.", "", $newquery);
          $newquery=ereg_replace("&showsel=.", "", $newquery);
          $newquery=ereg_replace("&clearlist=.", "", $newquery);
        }

        $selTxt="";
        if(count($selEntries)>0)
          $selTxt=" (".count($selEntries)." selected)";

        $pageTxt=array();
        if($entries_found > $MAX_ENTRIES)
        {
          $pageHtml=getPageHTML($SCRIPT_NAME."?".$newquery, $start, $entries_found, $MAX_ENTRIES);
          $pageTxt=array_merge(array(html_br()), $pageHtml);
        }

        $table1->push_row(
          create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"), 
                    html_img_href($playpre."irmp3cmd.php?comand=playsel$postfix", "images/play.gif", '', '', 0, "[Play]")),
          create_TD(array("CLASS"=>"Navigation", "ALIGN"=>"CENTER", "COLSPAN"=>"$colspan"),
                    create_B(array_merge(array("Entries ". ($start+1) ." - $end of $entries_found".$selTxt), $pageTxt))),
          create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"),
		    html_img_href($pleditpre."manage/pledit.php?plid=-1$postfix", "images/edit.gif", '', '', 0, "[Edit]")));

        $newquery=ereg_replace("sortby=.*&", "", $newquery);
        $newquery=ereg_replace("&sortby=.*$", "", $newquery);  // in case we are at the end of the query
        $newquery=$SCRIPT_NAME . "?" . $newquery . "&sortby=";

        $isortby=ereg_replace($sortdir, $isortdir, $sortby);
        $indirquery=$prefix . escape4JS($newquery . $isortby) . $postfix;

        $sortimg=ereg_replace("ASC", "images/down.gif", $isortdir);
        $sortimg=ereg_replace("DESC", "images/up.gif", $sortimg);
 
        $sortalt=ereg_replace("ASC", "[Down]", $isortdir);
        $sortalt=ereg_replace("DESC", "[UP]", $isortdir);
        $newquery=$prefix . escape4JS($newquery);

        $selAllAttr=array();
        if($HAS_JS)
          $selAllAttr["ONCLICK"]="javascript:selectEntries()";

        if($addentries==1)
          $selAllAttr[]="CHECKED";

	$t1r1=new TRtag();
        $t1r1->push(
           create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"),
		     form_checkbox("selall", NULL, $selAllAttr)),
           create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%", "ALIGN"=>"CENTER"),
		     html_img_href("$indirquery", "$sortimg", '', '', 0, "$sortalt")),
	   create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%")));

        if($SHOW_COLUMN["artist"])
           $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"20%", "ALIGN"=>"CENTER"),
	               html_b(html_a("$newquery"."artist+$sortdir%2Ctitle+$sortdir%2Calbum+$sortdir%2Ctrack+$sortdir$postfix", "Artist"))));
	   
        if($SHOW_COLUMN["composer"])
          $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"20%", "ALIGN"=>"CENTER"),
                      html_b(html_a("$newquery"."composer+$sortdir%2Cartist+$sortdir%2Ctitle+$sortdir%2Calbum+$sortdir%2Ctrack+$sortdir$postfix", "Composer"))));

        if($SHOW_COLUMN["title"])
          $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"30%", "ALIGN"=>"CENTER"),
                      html_b(html_a("$newquery"."title+$sortdir%2Cartist+$sortdir%2Calbum+$sortdir%2Ctrack+$sortdir$postfix", "Title"))));

        if($SHOW_COLUMN["album"])
          $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"20%", "ALIGN"=>"CENTER"),
                      html_b(html_a("$newquery"."album+$sortdir%2Ctrack+$sortdir%2Cartist+$sortdir%2Ctitle+$sortdir$postfix", "Album"))));
      }
      else  // no need for sorting links, cause we have only one entry 
      {
        $t1r1=new TRtag();
        $t1r1->push(
           create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%")),
           create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%")));

        if($SHOW_COLUMN["artist"])
           $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"20%", "ALIGN"=>"CENTER"),
                       html_b("Artist")));

        if($SHOW_COLUMN["composer"])
          $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"20%", "ALIGN"=>"CENTER"),
                      html_b("Composer")));

        if($SHOW_COLUMN["title"])
          $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"30%", "ALIGN"=>"CENTER"),
                      html_b("Title")));

        if($SHOW_COLUMN["album"])
          $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"20%", "ALIGN"=>"CENTER"),
                      html_b("Album")));
      }

      if($SHOW_COLUMN["length"])
        $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"5%"), html_b("Length")));

      if($SHOW_COLUMN["kbps"])
        $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"5%"), html_b("kbps")));

      if($SHOW_COLUMN["size"])
        $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"5%"), html_b("Size")));

      if($SHOW_COLUMN["file"])
        $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"), html_b("File")));

      if($SHOW_COLUMN["edit"])
        $t1r1->push(create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"), html_b("Edit")));

      $table1->push($t1r1);

      // Get results from database
      $visEntries=array();
      $rows=$start;
      while($rows < $end)
      {
        $song=pg_fetch_object($result, $rows);

        // Encode URLs
        $url=rawurlencode($song->file);
        $url=$LINKROOT . $song->file;

        $song->fileurl=rawurlencode($MP3ROOT . $song->file);
        $uartist=rawurlencode($song->artist);
        $ucomposer=rawurlencode($song->composer);
        $utitle=rawurlencode($song->title);
        $ualbum=rawurlencode($song->album);

        $ftime=formatSeconds($song->length);
        $fsize=formatSize($song->filesize);

        // convert html special chars where needed
        $song->artist=htmlspecialchars($song->artist);
        $song->composer=htmlspecialchars($song->composer);
        $song->title=htmlspecialchars($song->title);
        $song->album=htmlspecialchars($song->album);

        // create a table row for this entry 
        $style="Entry";
        if(strlen($song->album)==0 && $song->track==false)
          $style="NoEntry";

        if($song->track==false)
          $track="";
        else
          $track=" [$song->track]";

        $t1r1=new TRtag();

        $check=array();
        if(in_array($song->id, $selEntries))
        {
          $check[]="CHECKED";
          $visEntries[]=$song->id;
        }

        if($isSearch || $entries_found > 1)
          $t1r1->push(create_TD(array("CLASS"=>"Column"), form_checkbox("i_$song->id", "1", $check)));

        $t1r1->push(
          create_TD(array_merge(array("CLASS"=>"Number", "ALIGN"=>"RIGHT"), $ENTRY_ATTR),
	            html_a("info.php?id=$song->id", $rows+1, NULL, "_blank")),
          create_TD(array_merge(array("CLASS"=>"Button"), $ENTRY_ATTR),
	            html_img_href("irmp3cmd.php?comand=play&file=$song->fileurl", "images/play.gif", '', '', 0, "[Play]", NULL, "player")));

        if($SHOW_COLUMN["artist"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Entry"), $ENTRY_ATTR),
		      html_a($prefix."search.php?artist=$uartist$postfix", $song->artist , NULL, "muwed")));

        if($SHOW_COLUMN["composer"])
        {
          $cstyle="Entry";
          if(strlen($song->composer)==0)
            $cstyle="NoEntry";

          $t1r1->push(
            create_TD(array_merge(array("CLASS"=>"$cstyle"), $ENTRY_ATTR),
		      html_a($prefix."search.php?composer=$ucomposer$postfix", $song->composer, NULL, "muwed")));
        }

        if($SHOW_COLUMN["title"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Entry"), $ENTRY_ATTR),
		      html_a("plsearch.php?songid=$song->id", $song->title, NULL, "muwed")));

        if($SHOW_COLUMN["album"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"$style"), $ENTRY_ATTR),
 		      html_a($prefix."search.php?album=$ualbum$postfix", $song->album.$track, NULL, "muwed")));

        if($SHOW_COLUMN["length"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Entry", "ALIGN"=>"RIGHT"), $ENTRY_ATTR), "$ftime"));

        if($SHOW_COLUMN["kbps"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Entry", "ALIGN"=>"RIGHT"), $ENTRY_ATTR), "$song->bitrate"));

        if($SHOW_COLUMN["size"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Entry", "ALIGN"=>"RIGHT"), $ENTRY_ATTR), "$fsize"));

        if($SHOW_COLUMN["file"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Button"), $ENTRY_ATTR),
		      html_img_href("$url", "images/download.gif", '', '', 0, "[Download]")));

        if($SHOW_COLUMN["edit"])
          $t1r1->push(create_TD(array_merge(array("CLASS"=>"Button"), $ENTRY_ATTR),
		      html_img_href("manage/edit.php?id=$song->id", "images/edit.gif", '', '', 0, "[Edit]", NULL, "_blank")));
        $table1->push($t1r1);

        $rows++;
      }

      if($entries_found > $MAX_ENTRIES)
      {
        $table1->push_row(
          create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"),
		    html_img_href($playpre."irmp3cmd.php?comand=playsel$postfix", "images/play.gif", '', '', 0, NULL, $playTarget)),
	  create_TD(array("CLASS"=>"Navigation", "ALIGN"=>"MIDDLE", "COLSPAN"=>"$colspan"),
		    create_B($pageHtml)),
	  create_TD(array("CLASS"=>"Column", "WIDTH"=>"1%"),
		    html_img_href("irmp3cmd.php", "images/edit.gif", '', '', 0, "[Edit]", NULL,  "player")));
      }
   
      $selEntries=array_diff($selEntries, $visEntries);	// remove all visible entries from list
    }
  }

  return $table1;
}

function getPageHTML($query, $start, $num, $max)
{
  global $HAS_JS;

  $query=ereg_replace("start=.*&", "", $query);
  $query=ereg_replace("&start=.*^", "", $query);  // in case we are at the end of the query
  $query=$query . "&start=";

  $prefix="";
  $postfix="";
  if($HAS_JS)
  {
    $prefix="javascript:showMoreEntries('";
    $postfix="')";
  }

  $query=escape4JS($query);
  $html=array();
  if($num > $max)
  {
    $prev=$start - $max;
    if($prev < 0)
      $prev=-1;

    $next=$start + $max;
    if($next >= $num)
      $next=-1;

    if($prev >= 0)
    {
      $html[]=html_a("$prefix". $query ."0$postfix", "&lt;&lt");
      $html[]=" ";
      $html[]=html_a("$prefix". $query . $prev ."$postfix", "&lt");
    }
    else
      $html[]="&lt;&lt; &lt;";

    $cnt=1;
    for($i=0; $i<$num; $i+=$max)
    {
      if($i != $start)
      {
        $html[]=html_a("$prefix". $query . $i ."$postfix", "$cnt");
	$html[]=" ";
      }
      else
      {
        $html[]=create_B("$cnt", array("style"=>"color: #99aabb"));
      }

      $end=$i;
      $cnt++;
    }

    if($next >= 0)
    {
      $html[]=html_a("$prefix". $query . $next ."$postfix", "&gt;");
      $html[]=" ";
      $html[]=html_a("$prefix". $query . $end ."$postfix", "&gt;&gt;");
    }
    else
      $html[]="&gt; &gt;&gt;";
  }

  return $html;
}

?>
